#include "mamalleg.h"
#include "driver.h"
#include "ticker.h"
#include "be_sound_api.h"

static int num_used_opl;
int nominal_sample_rate;
int soundcard,usestereo;
int attenuation = 0;
int sampleratedetect = 1;
static int master_volume = 256;

static int stream_playing;
static INT16 *stream_cache_data;
static int stream_cache_len;
static int stream_cache_stereo;

int beos_init_sound(void)
{
	nominal_sample_rate = Machine->sample_rate;

	num_used_opl = 0;

	osd_set_mastervolume(attenuation);	/* set the startup volume */

	return 0;
}

void beos_shutdown_sound(void)
{
	close_be_sound();
}

#define NUM_BUFFERS 3	/* raising this number should improve performance with frameskip, */
						/* but also increases the latency. */

/* global sample tracking */
static double samples_per_frame;
static double samples_left_over;
static UINT32 samples_this_frame;

int osd_start_audio_stream(int stereo)
{
	if (stereo) stereo = 1;	/* make sure it's either 0 or 1 */

	stream_cache_stereo = stereo;

	/* determine the number of samples per frame */
	samples_per_frame = (double)Machine->sample_rate / (double)Machine->drv->frames_per_second;

	/* compute how many samples to generate this frame */
	samples_left_over = samples_per_frame;
	samples_this_frame = (UINT32)samples_left_over;
	samples_left_over -= (double)samples_this_frame;

	if (Machine->sample_rate == 0) return 0;

	if(!open_be_sound( stereo+1, nominal_sample_rate, samples_this_frame, NUM_BUFFERS))
		return 0;

	stream_playing = 1;

	return samples_this_frame;
}

void osd_stop_audio_stream(void)
{
	if (Machine->sample_rate == 0) return;

	close_be_sound();

	stream_playing = 0;
}

static void updateaudiostream(INT16* buffer, int samples)
{
	if (!stream_playing) return;	/* error */
	if(stream_cache_stereo){
		copy_be_sound_buffer( (void*)buffer, samples*4);
	}
	else{
		copy_be_sound_buffer( (void*)buffer, samples*2);
	}
}

int osd_update_audio_stream(INT16 *buffer)
{
	stream_cache_data = buffer;
	stream_cache_len = samples_this_frame;

	/* compute how many samples to generate next frame */
	samples_left_over += samples_per_frame;
	samples_this_frame = (UINT32)samples_left_over;
	samples_left_over -= (double)samples_this_frame;

	/* update audio stream */
	if (Machine->sample_rate != 0 && stream_cache_data != 0){
		profiler_mark(PROFILER_MIXER);
		updateaudiostream( buffer, samples_this_frame);
		profiler_mark(PROFILER_END);
	}

	return samples_this_frame;
}

int beos_update_audio(void)
{
	return 0;
}

/* attenuation in dB */
void osd_set_mastervolume(int _attenuation)
{
	float volume;

	if (_attenuation > 0) _attenuation = 0;
	if (_attenuation < -32) _attenuation = -32;

	attenuation = _attenuation;

 	volume = 256.0;	/* range is 0-256 */
	while (_attenuation++ < 0)
		volume /= 1.122018454;	/* = (10 ^ (1/20)) = 1dB */

	master_volume = volume;
	
	set_be_sound_volume( master_volume);
}
int osd_get_mastervolume(void)
{
	return	attenuation;
}

void osd_sound_enable(int enable_it)
{
	set_be_sound_enable( enable_it);
}

void osd_opl_control(int chip,int reg)
{
}
void osd_opl_write(int chip,int data)
{
}
