#include <MediaKit.h>
#include "be_sound_api.h"

CCirculationIO::CCirculationIO(size_t numBytes)
	: BDataIO()
	, f_locker( "locker", true)
	, f_buffer( (uint8*)malloc( numBytes))
	, f_buffer_size( numBytes)
	, f_w_pos( 0)
	, f_r_pos( 0)
	, f_n_pos( 0)
{
}
CCirculationIO::~CCirculationIO()
{
	free( f_buffer);
}
ssize_t	CCirculationIO::Read(void *buffer, size_t size)
{
	ssize_t	result = 0;
	if(this->f_locker.Lock()){
		//
		int32	red = (int32)size;
		if(this->f_n_pos<=0){
			this->f_locker.Unlock();
			return	result;	
		}
		if(this->f_n_pos<red) red = this->f_n_pos;
		result = (ssize_t)red;
		this->f_n_pos -= red;
		//
		uint8*	dst = (uint8*)buffer;
		int32	tmp = this->f_buffer_size - this->f_r_pos;
		if(tmp<red){
			memcpy( dst, this->f_buffer + this->f_r_pos, tmp);
			red -= tmp;
			dst += tmp;
			memcpy( dst, this->f_buffer, red);	
			this->f_r_pos = red;
		}
		else{
			memcpy( dst, this->f_buffer + this->f_r_pos, red);
			this->f_r_pos += red;
		}
		this->f_locker.Unlock();
	}	
	return	result;
}
ssize_t	CCirculationIO::Write(const void *buffer, size_t size)
{
	ssize_t	result = 0;
	if(this->f_locker.Lock()){
		//
		int32	rem = this->f_buffer_size - this->f_n_pos;
		int32	wri = (int32)size;
		if(rem<=0){
			this->f_locker.Unlock();
			return	result;	
		}
		if(rem<wri) wri = rem;
		result = (ssize_t)wri;
		this->f_n_pos += wri;
		//
		uint8*	src = (uint8*)buffer;
		int32	tmp = this->f_buffer_size - this->f_w_pos;
		if(tmp<wri){
			memcpy( this->f_buffer + this->f_w_pos, src, tmp);
			wri -= tmp;
			src += tmp;
			memcpy( this->f_buffer, src, wri);			
			this->f_w_pos = wri;
		}
		else{
			memcpy( this->f_buffer + this->f_w_pos, src, wri);
			this->f_w_pos += wri;
		}
		this->f_locker.Unlock();
	}
	return	result;
}

CMAMESound::CMAMESound(size_t inBufferFrameCount, const gs_audio_format * format, size_t inBufferCount)
	: BStreamingGameSound(inBufferFrameCount,format,inBufferCount)
{
	int32	buffer_size = (int32)(format->frame_rate * format->channel_count);
	switch( format->format){
		case gs_audio_format::B_GS_U8:break;
		case gs_audio_format::B_GS_S16:buffer_size *= 2;break;
	}
	this->f_buffer = new CCirculationIO( buffer_size);
	this->SetStreamHook( this->Callback, NULL);
}
CMAMESound::~CMAMESound()
{
	delete	this->f_buffer;
}
void	CMAMESound::Read(void *buffer, size_t size)
{
	if(this->Lock()){
		int	result = (int)this->f_buffer->Read( buffer, size);
		if(result<size){
			memset( ((uint8*)buffer) + result, 0, (int)size - result);
		}
		this->Unlock();
	}
}
void	CMAMESound::Write(void *buffer, size_t size)
{
	this->f_buffer->Write( buffer, size);
}
void	CMAMESound::Callback(void *cookie, void *inBuffer, size_t byteCount, BStreamingGameSound *object)
{
	((CMAMESound*)object)->Read( inBuffer, byteCount);
}

//	--------------------	

static	CMAMESound*	g_sound_device = NULL;
static	int			g_sound_enable = 0;
static	int			g_sound_volume = 0;

int	open_be_sound(int channel, int rate, int sample_per_frame, int num_buffer)
{
	gs_audio_format	format;
	// 二重処理防止
	if(g_sound_device)	close_be_sound();
	// BPushGameSound作成
	format.frame_rate		= rate;
	format.channel_count	= channel;
	format.format			= gs_audio_format::B_GS_S16;
	format.byte_order		= B_MEDIA_LITTLE_ENDIAN;
	format.buffer_size		= 0;
	g_sound_device = new CMAMESound( sample_per_frame, &format, num_buffer);
	if ( g_sound_device->InitCheck()!=B_OK){
		return 0;
	}
	g_sound_device->SetGain(1.0f);
	g_sound_device->StartPlaying();
	// 初期設定
	set_be_sound_enable(   1);
	set_be_sound_volume( 256);
	return	1;
}

void	close_be_sound(void)
{
	if(!g_sound_device)return;
	g_sound_device->StopPlaying();
	delete	g_sound_device;
	g_sound_device	 = NULL;
}

void	copy_be_sound_buffer(void** buffer, int size)
{
	if(g_sound_device){
		g_sound_device->Write( buffer, size);
	}
}

void	set_be_sound_enable(int enable_it)
{
	g_sound_enable = enable_it;
	if(g_sound_device)
		g_sound_device->SetGain( (float)(g_sound_enable * g_sound_volume) / 256.0f);
}

void	set_be_sound_volume(int volume)
{
	g_sound_volume = volume;
	if(g_sound_device)
		g_sound_device->SetGain( (float)(g_sound_enable * g_sound_volume) / 256.0f);
}
int	get_be_sound_volume(void)
{
	return	g_sound_volume;
}
